<?php

namespace Drupal\captchetat\Controller;

use Drupal\captchetat\CaptchetatApiHelper;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Controller\ControllerBase;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;

/**
 * This controller provides different connections to API CaptchEtat.
 */
class CaptchetatController extends ControllerBase {

  /**
   * The config factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * Constructs a new CaptchetatController object.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The config factory.
   */
  public function __construct(ConfigFactoryInterface $config_factory) {
    $this->configFactory = $config_factory;
  }

  /**
   * Returns if API is up or down.
   *
   * @return \Symfony\Component\HttpFoundation\Response
   *   HTTP Response.
   */
  public function healthCheck() {

    $token = $this->generateApiToken();
    $config = $this->configFactory->get('captchetat.settings');

    $response = CaptchetatApiHelper::healthCheck($token, $config);

    return new Response($response);
  }

  /**
   * Returns the captchetat token.
   *
   * @return string
   *   token provided by API CaptchEtat.
   */
  public function generateApiToken() {
    $config = $this->configFactory->get('captchetat.settings');
    return CaptchetatApiHelper::generateApiToken($config);
  }

  /**
   * Gets Captcha from API Captchetat.
   *
   * @param \Symfony\Component\HttpFoundation\Request $request
   *   The request object.
   *
   * @return \Symfony\Component\HttpFoundation\Response
   *   HTTP Response for the CaptchEtat element.
   */
  public function getCaptcha(Request $request) {
    $token = $this->generateApiToken();
    $config = $this->configFactory->get('captchetat.settings');
    $request_uri = $request->getRequestUri();
    $query_params = explode('?', $request_uri)[1];
    $url = $config->get('api_url') . '/piste/captcha/simple-captcha-endpoint?' . $query_params;
    $ch = curl_init();
    $headers = [
      "Authorization: Bearer $token",
    ];

    curl_setopt($ch, CURLOPT_URL, $url);
    curl_setopt($ch, CURLOPT_RETURNTRANSFER, TRUE);
    curl_setopt($ch, CURLOPT_HTTPHEADER, $headers);

    $response = curl_exec($ch);

    if (curl_errno($ch)) {
      echo 'Curl error: ' . curl_error($ch);
    }

    curl_close($ch);

    return new Response($response);
  }

  /**
   * Validates the user submission.
   *
   * @param \Symfony\Component\HttpFoundation\Request $request
   *   The request object.
   *
   * @return \Symfony\Component\HttpFoundation\Response
   *   HTTP Response for validation wheter the user entered the correct code.
   */
  public function validationCaptcha(Request $request) {

    $payload = $request->getContent();
    $decoded_payload = json_decode($payload, TRUE);
    $captcha_id = $decoded_payload['captchaId'];
    $user_entered_captcha_code = $decoded_payload['userEnteredCaptchaCode'];

    $token = $this->generateApiToken();

    $config = $this->configFactory->get('captchetat.settings');

    $response = CaptchetatApiHelper::validationCaptcha(
      $captcha_id, $user_entered_captcha_code, $token, $config);

    return new Response($response);
  }

}
